from pickart.colour_formats import ColourFormats


class Colour:
    def __init__(self, colour: tuple[int, ...]):
        """Initialize colour.

        Args:
            colour (tuple[int, ...]): tuple with 3 or 4 integers that represent colour - r, g, b, a. Alpha is optional
        """

        self.colour: tuple[int, ...] = colour[:4]
        self.fmt = ColourFormats(len(self.colour))
        if self.fmt == ColourFormats.RGBA and self.colour[3] != 255:
            self.colour = (*self.colour[:3], 255)

        self._generate_grayscale()

    def _generate_grayscale(self):
        colour = sum(self.colour[:3]) // 3
        if colour == 0:
            colour = 10
        self.grayscale = (
            (colour, colour, colour, 255)
            if self.fmt == ColourFormats.RGBA
            else (colour, colour, colour)
        )

    @staticmethod
    def check_colour(colour: tuple[int, ...], fmt: ColourFormats) -> bool:
        return len(colour) == fmt.value and all(
            (c <= 255 and c >= 0 and isinstance(c, int) for c in colour)
        )

    def __repr__(self) -> str:
        colour = self.colour
        fmt = self.fmt.name
        return f"Colour({colour=},  {fmt=})"



import unittest
import pickle
from pathlib import Path
from hashlib import sha1
from shutil import copyfile
from gzip import GzipFile

from pickart import PickartFile, PickartFileData
from pickart.errors import BadPickartFile, BadPixelFormat
from pickart.pickart_file import set_stdo


FILES_DIR = Path("tests", "pickart")
SAVING_TESTS_DIR = Path(FILES_DIR, "saving_tests")
PICKART_FILE_WITH_EXTERNAL_OBJ = Path(FILES_DIR, "with_external_obj.pickart")
PICKART_FILE_WITH_WRONG_FORMAT = Path(FILES_DIR, "with_wrong_format.pickart")
PICKART_FILE_WITH_WRONG_VERSION = Path(FILES_DIR, "with_wrong_version.pickart")
PICKART_FILE_WITH_WRONG_SIZE = Path(FILES_DIR, "with_wrong_size.pickart")
PICKART_FILE_WITH_WRONG_PIXEL_FORMAT = Path(
    FILES_DIR, "with_wrong_pixel_format.pickart"
)
NORMAL_PICKART_FILE = Path(FILES_DIR, "normal_file.pickart")
FILE_FOR_SAVING_TESTS = Path(SAVING_TESTS_DIR, "save_test.pickart")


set_stdo(lambda x: None)
"""Standtart output for file loader. It is empty function that is used instead of standart `print`."""


def get_gzip_file_hash(file_path: Path) -> str:
    with GzipFile(file_path, "rb") as file:
        return sha1(file.read()).hexdigest()


class TestPickartFile(unittest.TestCase):
    def test_loading_pickart_file_with_external_obj(self):
        pickart_file = PickartFile(PICKART_FILE_WITH_EXTERNAL_OBJ)
        self.assertEqual(len(pickart_file.errors), 1)
        self.assertIsInstance(pickart_file.errors[0], pickle.UnpicklingError)
        self.assertEqual(pickart_file.filepath, PICKART_FILE_WITH_EXTERNAL_OBJ)
        self.assertFalse(pickart_file.valid)

    def test_loading_pickart_file_with_wrong_format(self):
        pickart_file = PickartFile(PICKART_FILE_WITH_WRONG_FORMAT)
        self.assertEqual(len(pickart_file.errors), 1)
        self.assertIsInstance(pickart_file.errors[0], TypeError)
        self.assertEqual(pickart_file.filepath, PICKART_FILE_WITH_WRONG_FORMAT)
        self.assertFalse(pickart_file.valid)

    def test_loading_pickart_file_with_wrong_version(self):
        pickart_file = PickartFile(PICKART_FILE_WITH_WRONG_VERSION)
        self.assertEqual(len(pickart_file.errors), 1)
        self.assertIsInstance(pickart_file.errors[0], BadPickartFile)
        self.assertEqual(pickart_file.filepath, PICKART_FILE_WITH_WRONG_VERSION)
        self.assertFalse(pickart_file.valid)

    def test_loading_pickart_file_with_wrong_size(self):
        pickart_file = PickartFile(PICKART_FILE_WITH_WRONG_SIZE)
        self.assertEqual(len(pickart_file.errors), 1)
        self.assertIsInstance(pickart_file.errors[0], BadPickartFile)
        self.assertEqual(pickart_file.filepath, PICKART_FILE_WITH_WRONG_SIZE)
        self.assertFalse(pickart_file.valid)

    def test_loading_pickart_file_with_wrong_pixel_format(self):
        pickart_file = PickartFile(PICKART_FILE_WITH_WRONG_PIXEL_FORMAT)
        self.assertEqual(len(pickart_file.errors), 1)
        self.assertIsInstance(pickart_file.errors[0], BadPixelFormat)
        self.assertEqual(pickart_file.filepath, PICKART_FILE_WITH_WRONG_PIXEL_FORMAT)
        self.assertFalse(pickart_file.valid)

    def test_loading_normal_file(self):
        pickart_file = PickartFile()
        self.assertIs(pickart_file.filepath, None)
        self.assertFalse(pickart_file.valid)
        pickart_file.load(NORMAL_PICKART_FILE)
        self.assertEqual(pickart_file.filepath, NORMAL_PICKART_FILE)
        self.assertTrue(pickart_file.valid)

    def test_saving_file_from_file_data(self):
        file_data = PickartFileData({"size": (1, 1), "version": 1})
        pickart_file = PickartFile(file_data=file_data)
        self.assertIs(pickart_file.filepath, None)
        self.assertFalse(pickart_file.valid)
        pickart_file.save(FILE_FOR_SAVING_TESTS)
        self.assertEqual(pickart_file.filepath, FILE_FOR_SAVING_TESTS)
        self.assertTrue(pickart_file.valid)

    def test_saving_file(self):
        NORMAL_PICKART_FILE_FOR_SAVING = Path(
            SAVING_TESTS_DIR, NORMAL_PICKART_FILE.name
        )

        copyfile(NORMAL_PICKART_FILE, NORMAL_PICKART_FILE_FOR_SAVING)

        pickart_file = PickartFile(NORMAL_PICKART_FILE_FOR_SAVING)
        pickart_file.save(FILE_FOR_SAVING_TESTS)
        self.assertEqual(pickart_file.filepath, FILE_FOR_SAVING_TESTS)
        self.assertTrue(pickart_file.valid)
        self.assertEqual(
            get_gzip_file_hash(FILE_FOR_SAVING_TESTS),
            get_gzip_file_hash(NORMAL_PICKART_FILE_FOR_SAVING),
        )
        saved_pickart_file = PickartFile(FILE_FOR_SAVING_TESTS)
        self.assertEqual(len(saved_pickart_file.errors), 0)
        self.assertEqual(saved_pickart_file._data, pickart_file._data)


if __name__ == "__main__":
    unittest.main()



import pickle
from pathlib import Path
from gzip import GzipFile
from dataclasses import asdict
from typing import Optional, Callable

from pickart.errors import BadPickartFile, BadPixelFormat
from pickart.colour import Colour
from pickart.constatnts import PICKART_VERSON, strOrPath
from pickart.colour_formats import ColourFormats
from pickart.pickart_file_data import PickartFileData
from pickart.restricted_unpickler import load


def set_stdo(new_stdo: Callable):
    """Set new global stdo for Pickart files. `stdo` is used as standart output for errors that happend during file loading.

    Args:
        new_stdo (Callable): function with one positional argument. This argument is string with error message.

    Raises:
        ValueError: if argument `new_stdo` is not callable.
    """

    if not callable(new_stdo):
        raise ValueError(f"'new_stdo' must be callable.")
    PickartFile._STDO = new_stdo


class PickartFile:
    _STDO = print

    def __init__(
        self,
        filepath: Optional[strOrPath] = None,
        file_data: Optional[PickartFileData] = None,
        stdo: Optional[Callable] = None,
    ):
        """Load Pickart file. If file does not exist or is not Pickart format exception `will not` be raised to check if loading was successful use `valid` attribute.

        Args:
            filepath (Optional[strOrPath]): path to .pickart file.
            file_data (Optional[PickartFileData]): file data. Note if `filepath` is passed this argument will be ignored.
            stdo (Callable): function which will be called when error occur during file loading. Defaults to `print`. Can be changed via this argument, or globaly(for all Pickart instances) with set_stdo function.
        Raises:
            ValueError: if `filepath` or `file_data` has wrong type.
        """
        self.stdo = PickartFile._STDO
        if stdo is not None:
            self.stdo = stdo
        self._data = PickartFileData()
        self.filepath = None
        self.errors = []

        if file_data is not None:
            if not isinstance(file_data, PickartFileData):
                raise ValueError(
                    f"'file_data' must be pickart.PickartFileData type, not: {type(file_data)}."
                )
            self._data = file_data

        self._colour_palette: dict[int, Colour] = {}
        self.valid = False

        if filepath is not None:
            if not isinstance(filepath, (str, Path)):
                raise ValueError(
                    f"'filepath' must be str or pathlib.Path type, not: {type(filepath)}."
                )
            self.filepath = Path(filepath)
            self.load()

    def _load_file(self):
        try:
            with GzipFile(self.filepath) as file:
                self._data = PickartFileData(**load(file))
                version: int = self._data.info.get("version", None)
                if version != PICKART_VERSON:
                    raise BadPickartFile(f"File version: '{version}' is not supported.")
                if any(i <= 0 for i in self._data.info.get("size", (0, 0))):
                    raise BadPickartFile("Art size must be positive")

                self.fmt = ColourFormats(len(self._data.palette[0]))
                self.valid = True
        except Exception as error:
            self.errors.append(error)
            self.stdo(f"Cannot load file '{self.filepath}', error: {error}.")

    def load(self, filepath: Optional[strOrPath] = None):
        """Load file from filepath.

        Args:
            filepath (Optional[strOrPath]): new filepath, will override existing filepath.

        Raises:
            ValueError: if `filepath` has wrong type.
        """

        if filepath is not None:
            if not isinstance(filepath, (str, Path)):
                raise ValueError(
                    f"'filepath' must be str or pathlib.Path type, not: {type(filepath)}."
                )
            self.filepath = filepath

        self._load_file()

        if not self.valid:
            return

        for key, colour in enumerate(self._data.palette):
            if not Colour.check_colour(colour, self.fmt):
                self.valid = False
                error_msg = f"Bad pixel format in file: '{self.filepath}'."
                self.errors.append(BadPixelFormat(error_msg))
                self.stdo(error_msg)
                break

            self._colour_palette[key] = Colour(colour)

    def set_painted(self, x: int, y: int, colour_index: int) -> bool:
        """Paint pixel at `x`, `y` if `colour_index` is the same. Return `False` if pixel is already painted or `colour_index` is different, else return `True`."""

        if self._data.pixels[y][x][0] != colour_index or self._data.pixels[y][x][1]:
            return False
        self._data.pixels[y][x][1] = True
        return True

    def get_pixels(self) -> list[list]:
        return self._data.pixels

    def get_size(self) -> tuple[int, int]:
        return self._data.info["size"]

    def get_palette(self):
        return self._colour_palette

    def save(self, filepath: Optional[strOrPath] = None):
        """Save file to filepath.

        Args:
            filepath (Optional[strOrPath]): new filepath will, override existing filepath.

        Raises:
            ValueError: if `filepath` has wrong type.
        """

        if filepath is not None:
            if not isinstance(filepath, (str, Path)):
                raise ValueError(
                    f"'filepath' must be str or pathlib.Path type, not: {type(filepath)}."
                )
            self.filepath = filepath
        with GzipFile(self.filepath, "w") as file:
            pickle.dump(asdict(self._data), file)
        self.valid = True

    def __repr__(self) -> str:
        filepath = self.filepath
        if filepath is not None:
            filepath = str(filepath)

        valid = self.valid
        size = None
        if valid:
            size = self.get_size()
        return f"PickartFile({filepath=}, {size=}, {valid=})"



import os
import unittest
from pathlib import Path
from hashlib import sha1


from pickart import PickartFile
from pickart.converter import convert_to_pickart, convert_to_png

FILES_DIR = Path("tests", "converter")
OUTPUT_DIR = Path(FILES_DIR, "output")
INPUT_DIR = Path(FILES_DIR, "input")
NORMAL_PNG_FILE = Path(INPUT_DIR, "normal.png")
NORMAL_PICKART_FILE = Path(INPUT_DIR, "normal.pickart")
CONVERTED_NORMAL_PNG_FILE_HASH = "625dbe2dd88c1a293c3de87166f087036642cb2d"


class TestConverter(unittest.TestCase):
    def testPNGtoPickartConvertion(self):
        expected_name = Path(OUTPUT_DIR, "normal.pickart")
        convert_to_pickart(NORMAL_PNG_FILE, OUTPUT_DIR)
        self.assertTrue(expected_name.is_file())
        pickart_file = PickartFile(expected_name)
        self.assertTrue(pickart_file.valid)
        os.remove(expected_name)

    def testPNGtoPickartConvertionWithDifferentName(self):
        expected_name = Path(OUTPUT_DIR, "temp_res.pickart")
        convert_to_pickart(NORMAL_PNG_FILE, OUTPUT_DIR, "temp_res")
        self.assertTrue(expected_name.is_file())
        pickart_file = PickartFile(expected_name)
        self.assertTrue(pickart_file.valid)
        os.remove(expected_name)

    def testPickartToPNG(self):
        expected_name = Path(OUTPUT_DIR, "normal.png")
        convert_to_png(NORMAL_PICKART_FILE, OUTPUT_DIR)
        self.assertTrue(expected_name.is_file())
        self.assertEqual(
            sha1(expected_name.read_bytes()).hexdigest(), CONVERTED_NORMAL_PNG_FILE_HASH
        )
        os.remove(expected_name)

    def testPickartToPNGWithDifferentName(self):
        expected_name = Path(OUTPUT_DIR, "temp_res.png")
        convert_to_png(NORMAL_PICKART_FILE, OUTPUT_DIR, "temp_res")
        self.assertTrue(expected_name.is_file())
        self.assertEqual(
            sha1(expected_name.read_bytes()).hexdigest(), CONVERTED_NORMAL_PNG_FILE_HASH
        )
        os.remove(expected_name)


if __name__ == "__main__":
    unittest.main()



from pathlib import Path

from argparse import ArgumentParser

from pickart.converter import convert_to_pickart
from pickart.converter import convert_to_png

parser = ArgumentParser()

parser.add_argument("-i", default="input", help="Input folder(default 'input')")
parser.add_argument(
    "-o",
    default="output",
    help="Output folder(default 'output'). If folder does not exist it will be created.",
)
parser.add_argument(
    "-m",
    choices=("to_pickart", "to_png"),
    default="to_pickart",
    help="Convertion mode(default 'to_pickart'). Note: mode 'to_pickart' converts only .png images.",
)

args = parser.parse_args()


def main():
    input_dir: Path = Path(args.i)
    output_dir: Path = Path(args.o)

    if not input_dir.is_dir():
        print(f"'{input_dir}' folder does not exist.")
        return

    if args.m == "to_pickart":
        for filename in input_dir.glob("*.png"):
            print(f"Converting '{filename}' to pickart")
            convert_to_pickart(filename, output_dir)
    elif args.m == "to_png":
        for filename in input_dir.glob("*.pickart"):
            print(f"Converting '{filename}' to png")
            convert_to_png(filename, output_dir)


main()



from setuptools import setup, find_packages

with open("LICENSE", "r", encoding="utf-8") as file:
    LICENSE = file.read()

with open("README.md", "r", encoding="utf-8") as file:
    LONG_DESCRIPTION = file.read()

setup(
    name="Pickart",
    version="1.0.0",
    packages=find_packages(),
    python_requires=">=3.9",
    description="This is helper package for game called 'Colouring art'",
    author="AntynK",
    license=LICENSE,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    install_requires=["pygame>=2.1.2"],
    url="https://github.com/AntynK/Pickart"
)



import struct
from io import BytesIO
from pathlib import Path
from typing import Optional

import pygame

from pickart import PickartFileData, PickartFile
from pickart.constatnts import PICKART_VERSON, strOrPath


def convert_to_pickart(
    filename: strOrPath, output_dir: strOrPath, converted_filename: Optional[str] = None
):
    """Convert `.png` file to `.pickart`.

    Args:
        filename (strOrPath): `.png` file path.
        output_dir (strOrPath): folder where converted file will be saved. If it does not exist it will be created.
        converted_filename (Optional[str]): name for converted file, `.pickart` extension will be added if needed.

    Raises:
        ValueError: if `filename` does not exist.
    """

    filename = Path(filename)
    output_dir = Path(output_dir)
    output_dir.mkdir(parents=True, exist_ok=True)

    if converted_filename is None:
        converted_filename = f"{filename.stem}.pickart"
    elif not converted_filename.endswith(".pickart"):
        converted_filename += ".pickart"

    if not filename.is_file():
        raise ValueError(f"'{filename}' does not exist.")

    image = pygame.image.load(filename)
    file_data = BytesIO(image.get_buffer().raw)  # type: ignore

    byte_size = image.get_bytesize()
    pixel_fmt = ">" + "B" * byte_size

    width, height = image.get_size()

    pixels = []
    palette = {}

    header = {
        "size": (width, height),
        "version": PICKART_VERSON,
    }

    for _ in range(height):
        row = []
        for _ in range(width):
            colour = struct.unpack_from(pixel_fmt, file_data.read(byte_size))
            if byte_size == 4 and colour[3] == 0:  # Transparent pixel
                row.append([None, False])
                continue

            if colour not in palette:
                palette[colour] = len(palette)

            colour_index = palette[colour]
            row.append([colour_index, False])

        pixels.append(row)

    result = PickartFileData(header, list(palette.keys()), pixels)
    output_filename = Path(output_dir, converted_filename)

    file = PickartFile(file_data=result)
    file.save(output_filename)
